package com.qmth.cqb.paper.service;

import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;
import com.qmth.cqb.paper.dao.PaperDetailUnitRepo;
import com.qmth.cqb.paper.dto.PaperDetailUnitExp;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.model.PaperDetail;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.question.dao.QuesRepo;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.question.service.QuesService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class PaperDetailUnitService {

    @Autowired
    PaperDetailUnitRepo paperDetailUnitRepo;

    @Autowired
    QuesService quesService;

    @Autowired
    QuesRepo quesRepo;

    @Autowired
    PaperService paperService;

    /**
     * 根据Id获得对应的试题对象
     *
     * @param id
     * @return
     */
    public Question getQuestionByPaperDetailUnitId(String id) {
        return paperDetailUnitRepo.findOne(id).getQuestion();
    }

    /**
     * 按ID查询小题
     *
     * @param id
     * @return
     */
    public PaperDetailUnit findById(String id) {
        return paperDetailUnitRepo.findOne(id);
    }

    /**
     * 保存小题
     *
     * @param pduExp
     * @return
     */
    public PaperDetailUnit savePaperDetailUnit(PaperDetailUnitExp pduExp) {
        PaperDetailUnit oldPdu = paperDetailUnitRepo.findOne(pduExp.getId());
        if (oldPdu.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
            Question updateQuestion = null;
            if (pduExp.getQuestion().getId().equals(oldPdu.getQuestion().getId())) {
                updateQuestion = oldPdu.getQuestion();
            }
            List<Double> subScoreList = oldPdu.getSubScoreList();
            //套题需要判断更新的对象是套题本身还是子题
            int index = 0;
            for (Question sub : oldPdu.getQuestion().getSubQuestions()) {
                index++;
                //检查子题分数，确保没有错误数据
                if (subScoreList.size() < index) {
                    subScoreList.add(0d);
                }
                if (sub.getId().equals(pduExp.getQuestion().getId())) {
                    //匹配到子题
                    updateQuestion = sub;
                    subScoreList.set(index - 1, pduExp.getScore());
                }
            }
            if (updateQuestion == null) {
                //匹配试题失败，参数错误直接返回
                return oldPdu;
            }
            //更新套题unit当前总分
            oldPdu.updateSubScore(subScoreList);
            if (updateQuestion.getId().equals(oldPdu.getQuestion().getId())) {
                //更新的是套题大题
                //oldPdu.setQuestion(updateQuestion);
            } else {
                //更新的是套题下的子题
            }
            //quesRepo.save(updateQuestion);// 暂时不支持修改套题内容

        } else {
            oldPdu.setScore(pduExp.getScore());
            oldPdu.setQuestion(pduExp.getQuestion());
            Question ques = pduExp.getQuestion();
            quesRepo.save(ques);// 同时要跟新小题里面的Qustion对象
        }
        paperService.formatPaper(oldPdu.getPaper());
        return paperDetailUnitRepo.save(oldPdu);
    }

    /**
     * 删除小题
     *
     * @param id
     * @return
     */
    public void deletePaperDetailUnit(String id) {
        Paper paper = paperDetailUnitRepo.findOne(id).getPaper();
        paperDetailUnitRepo.delete(id);
        paperService.formatPaper(paper);
    }

    /**
     * 根据大题查小题
     *
     * @param paperDetail
     * @return
     */
    public List<PaperDetailUnit> getUnitsByPaperDetail(PaperDetail paperDetail) {
        return paperDetailUnitRepo.findByPaperDetail(paperDetail);
    }

    /**
     * 根据大题集合删除小题
     *
     * @param paperDetails
     */
    public void deleteUnitsByPaperDetails(List<PaperDetail> paperDetails) {
        List<PaperDetailUnit> units = new ArrayList<PaperDetailUnit>();
        for (PaperDetail pd : paperDetails) {
            units.addAll(getUnitsByPaperDetail(pd));
        }
        paperDetailUnitRepo.delete(units);
    }
}
